@@ -841,6 +841,22 @@ void MDI_Set_World_Rank(int world_rank_in)
 }
 
 
+/*! \brief Get the rank of this process within the MPI intra-communicator for the current code
+ *
+ * This function is only used by the Fortran wrapper
+ *
+ */
+int MDI_Get_intra_rank(int intra_rank_out)
+{
+  if ( is_initialized == 0 ) {
+    mdi_error("MDI_Get_intra_rank called but MDI has not been initialized");
+    return 1;
+  }
+  code* this_code = get_code(current_code);
+  return this_code->intra_rank;
+}
+
+
 /*! \brief Register a node
  *
  * The function returns \p 0 on a success.
@@ -1628,9 +1644,14 @@ int MDI_MPI_set_world_comm(void* world_comm)
 int MDI_Launch_plugin(const char* plugin_name, const char* options, void* mpi_comm_ptr,
                       MDI_Driver_node_callback_t driver_node_callback,
                       void* driver_callback_object) {
+#if _MDI_PLUGIN_SUPPORT == 1
   int ret = library_launch_plugin(plugin_name, options, mpi_comm_ptr,
                                   driver_node_callback, driver_callback_object);
   return ret;
+#else
+  mdi_error("MDI_Launch_plugin was called, but this build of the MDI Library was built without plugin support.");
+  return 1;
+#endif
 }
 
 
