@@ -25,6 +25,7 @@ mdi_name_path = os.path.join( dir_path, "mdi_name" )
 mdi_name_file = open( mdi_name_path, "r" )
 mdi_name = mdi_name_file.read()
 mdi_path = os.path.join( dir_path, mdi_name )
+mdi_name_file.close()
 
 # load the MDI Library
 try:
@@ -466,7 +467,7 @@ def MDI_Get_python_plugin_mpi_world_ptr():
 # MDI_Init
 mdi.MDI_Init_with_options.argtypes = [ctypes.POINTER(ctypes.c_char)]
 mdi.MDI_Init_with_options.restype = ctypes.c_int
-def MDI_Init(arg1):
+def MDI_Init(arg1, arg2 = None):
     global world_comm
     global intra_code_comm
 
@@ -481,7 +482,6 @@ def MDI_Init(arg1):
         try:
             # Assume python_plugin_mpi_world_ptr points to a pointer
             __mdi_mpi_comm_ptr__ = ctypes.cast(python_plugin_mpi_world_ptr, ctypes.POINTER(ctypes.c_void_p))
-            ptr_type = ctypes.c_void_p
             handle_t = ctypes.c_void_p
             newobj = type(MPI.COMM_WORLD)()
             handle_old = __mdi_mpi_comm_ptr__.contents
@@ -490,8 +490,8 @@ def MDI_Init(arg1):
             __mdi_plugin_mpi_intra_comm__ = newobj
 
             # Confirm that the new MPI communicator works
-            world_size = __mdi_plugin_mpi_intra_comm__.Get_size()
-        except:
+            __mdi_plugin_mpi_intra_comm__.Get_size()
+        except Exception as e:
             # Assume python_plugin_mpi_world_ptr points to an int
             __mdi_mpi_comm_ptr__ = ctypes.cast(python_plugin_mpi_world_ptr, ctypes.POINTER(ctypes.c_int))
             handle_old_value = __mdi_mpi_comm_ptr__.contents.value
@@ -502,7 +502,7 @@ def MDI_Init(arg1):
             __mdi_plugin_mpi_intra_comm__ = newobj
 
             # Confirm that the new MPI communicator works
-            world_size = __mdi_plugin_mpi_intra_comm__.Get_size()
+            __mdi_plugin_mpi_intra_comm__.Get_size()
         comm = __mdi_plugin_mpi_intra_comm__
 
 
@@ -530,8 +530,6 @@ def MDI_Init(arg1):
     if use_mpi4py:
         world_comm = comm
         intra_code_comm = comm
-        mpi_communicator = MPI._addressof(comm)
-        mpi_communicator_ptr = ctypes.c_void_p(mpi_communicator)
 
         # send basic information about the MPI communicator to the MDI libarary
         mpi_rank = comm.Get_rank()
@@ -560,6 +558,10 @@ def MDI_MPI_get_world_comm():
     global intra_code_comm
     return intra_code_comm
 
+# Included only for backwards compatibility
+def MDI_Get_Intra_Code_MPI_Comm():
+    return MDI_MPI_get_world_comm()
+
 def MDI_MPI_set_world_comm(new_comm):
     global intra_code_comm
     intra_code_comm = new_comm
