@@ -23,6 +23,33 @@
  *                   Options describing the communication method used to connect to codes.
  */
 int general_init(const char* options) {
+  // If this is the first time MDI has initialized, initialize the method vector
+  if ( ! is_initialized ) {
+
+    vector_init(&methods, sizeof(method));
+
+    // Create method objects for each supported method
+    if ( enable_tcp_support() ) {
+      mdi_error("Unable to enable TCP support");
+      return 1;
+    }
+    if ( enable_mpi_support() ) {
+      mdi_error("Unable to enable MPI support");
+      return 1;
+    }
+#if _MDI_PLUGIN_SUPPORT == 1
+    if ( enable_plug_support() ) {
+      mdi_error("Unable to enable plugin support");
+      return 1;
+    }
+#endif
+    if ( enable_test_support() ) {
+      mdi_error("Unable to enable TEST support");
+      return 1;
+    }
+
+  }
+
   // If this is the first time MDI has initialized, initialize the code vector
   if ( ! is_initialized ) {
     vector_init(&codes, sizeof(code));
@@ -36,20 +63,14 @@ int general_init(const char* options) {
   char* strtol_ptr;
   int i, ret;
 
-  int mpi_initialized = 0;
-
   // values acquired from the input options
   char* role;
-  char* method;
-  char* hostname;
-  int port;
+  char* method_str;
   char* output_file;
   char* language_argument = ((char*)"");
   int has_role = 0;
   int has_method = 0;
   int has_name = 0;
-  int has_hostname = 0;
-  int has_port = 0;
   int has_plugin_path = 0;
   int has_output_file = 0;
 
@@ -83,7 +104,11 @@ int general_init(const char* options) {
 	return 1;
       }
       role = argv[iarg+1];
-      snprintf(this_code->role, NAME_LENGTH, "%s", role);
+      if ( strlen(role) > NAME_LENGTH ) {
+	mdi_error("Error in MDI_Init: Role option is larger than MDI_NAME_LENGTH");
+	return 1;
+      }
+      snprintf(this_code->role, strlen(role)+1, "%s", role);
       has_role = 1;
       iarg += 2;
     }
@@ -93,7 +118,7 @@ int general_init(const char* options) {
 	mdi_error("Error in MDI_Init: Argument missing from -method option");
 	return 1;
       }
-      method = argv[iarg+1];
+      method_str = argv[iarg+1];
       has_method = 1;
       iarg += 2;
     }
@@ -107,7 +132,7 @@ int general_init(const char* options) {
 	mdi_error("Error in MDI_Init: Name argument length exceeds MDI_NAME_LENGTH");
 	return 1;
       }
-      snprintf(this_code->name, NAME_LENGTH, "%s", argv[iarg+1]);
+      snprintf(this_code->name, strlen(argv[iarg+1])+1, "%s", argv[iarg+1]);
       has_name = 1;
       iarg += 2;
     }
@@ -118,7 +143,6 @@ int general_init(const char* options) {
 	return 1;
       }
       hostname = argv[iarg+1];
-      has_hostname = 1;
       iarg += 2;
     }
     //-port
@@ -128,7 +152,6 @@ int general_init(const char* options) {
 	return 1;
       }
       port = strtol( argv[iarg+1], &strtol_ptr, 10 );
-      has_port = 1;
       iarg += 2;
     }
     //-ipi
@@ -152,7 +175,11 @@ int general_init(const char* options) {
 	mdi_error("Error in MDI_Init: Argument missing from -plugin_path option");
 	return 1;
       }
-      snprintf(this_code->plugin_path, PLUGIN_PATH_LENGTH, "%s", argv[iarg+1]);
+      if ( strlen(argv[iarg+1]) > PLUGIN_PATH_LENGTH ) {
+	mdi_error("Error in MDI_Init: Plugin path is larger than PLUGIN_PATH_LENGTH");
+	return 1;
+      }
+      snprintf(this_code->plugin_path, strlen(argv[iarg+1])+1, "%s", argv[iarg+1]);
       has_plugin_path = 1;
       iarg += 2;
     }
@@ -198,67 +225,31 @@ int general_init(const char* options) {
     return 1;
   }
 
-  // if using the MPI method, check if MPI has been initialized
-  int mpi_init_flag = 0;
-  if ( strcmp(method, "MPI") == 0 && this_code->language != MDI_LANGUAGE_PYTHON ) {
-
-    ret = MPI_Initialized(&mpi_init_flag);
-    if ( ret != 0 ) {
-      mdi_error("Error in MDI_Init: MPI_Initialized failed");
-      return ret;
-    }
-
-    if ( mpi_init_flag == 0 ) {
-
-      // initialize MPI
-      int mpi_argc = 0;
-      char** mpi_argv;
-      ret = MPI_Init( &mpi_argc, &mpi_argv );
-      if ( ret != 0 ) {
-	mdi_error("Error in MDI_Init: MPI_Init failed");
-	return ret;
-      }
-
-      // confirm that MPI is now initialized
-      // if it isn't, that indicates that the MPI stubs are being used
-      ret = MPI_Initialized(&mpi_init_flag);
-      if ( ret != 0 ) {
-	mdi_error("Error in MDI_Init: MPI_Initialized failed");
-	return ret;
-      }
-      if ( mpi_init_flag == 0 ) {
-	mdi_error("Error in MDI_Init: Failed to initialize MPI. Check that the MDI Library is linked to an MPI library.");
-	return 1;
-      }
-
-      // get the world_comm
-      mdi_mpi_comm_world = MPI_COMM_WORLD;
-      initialized_mpi = 1;
-
-    }
 
+  // determine the method id of the method selected by the user
+  if ( strcmp(method_str, "TCP") == 0 ) {
+    selected_method_id = MDI_TCP;
   }
-
-  // get the appropriate MPI communicator to use
-  MPI_Comm mpi_communicator;
-  ret = MPI_Initialized(&mpi_init_flag);
-  if ( ret != 0 ) {
-    mdi_error("Error in MDI_Init: MPI_Initialized failed");
-    return ret;
+  else if ( strcmp(method_str, "MPI") == 0 ) {
+    selected_method_id = MDI_MPI;
   }
-  if ( mpi_init_flag == 0 ) {
-    mpi_communicator = 0;
+  else if ( strcmp(method_str, "LINK") == 0 || strcmp(method_str, "PLUG") == 0 ) {
+    selected_method_id = MDI_LINK;
+  }
+  else if ( strcmp(method_str, "TEST") == 0 ) {
+    selected_method_id = MDI_TEST;
   }
   else {
-    if ( this_code->language == MDI_LANGUAGE_PYTHON ) {
-      mpi_communicator = 0;
-    }
-    else {
-      mpi_communicator = MPI_COMM_WORLD;
-      MPI_Comm_rank(mpi_communicator, &world_rank);
-      MPI_Comm_size(mpi_communicator, &world_size);
-      this_code->intra_rank = world_rank;
-    }
+    mdi_error("Error in MDI_Init: Method not recognized");
+    return 1;
+  }
+  method* selected_method = get_method(selected_method_id);
+
+  // ensure that a valid role has been provided
+  if ( strcmp(this_code->role, "DRIVER") != 0 &&
+       strcmp(this_code->role, "ENGINE") != 0 ) {
+    mdi_error("Error in MDI_Init: Role not recognized");
+    return 1;
   }
 
   // redirect the standard output
@@ -266,14 +257,6 @@ int general_init(const char* options) {
     freopen(output_file, "w", stdout);
   }
 
-  // if the method is not LIB, ensure that MDI has not been previously initialized
-  if ( strcmp(method, "LINK") != 0 ) {
-    if ( is_initialized == 1 ) {
-      mdi_error("MDI_Init called after MDI was already initialized");
-      return 1;
-    }
-  }
-
   // ensure that the name of this code is not the same as the name of any of the other codes
   for (i = 0; i < codes.size; i++) {
     if ( i != current_code ) {
@@ -285,15 +268,8 @@ int general_init(const char* options) {
     }
   }
 
-  // Check if this is an engine being used as a library
-  if (strcmp(this_code->role, "ENGINE") == 0) {
-    if ( strcmp(method, "LINK") == 0 ) {
-      this_code->is_library = 1;
-    }
-  }
-
   // ensure that at most one driver has been initialized
-  if (strcmp(this_code->role, "DRIVER") == 0) {
+  if ( strcmp(this_code->role, "DRIVER") == 0 ) {
     for (i = 0; i < codes.size; i++) {
       if ( i != current_code ) {
 	code* other_code = vector_get(&codes, i);
@@ -305,86 +281,22 @@ int general_init(const char* options) {
     }
   }
 
-  // determine whether the intra-code MPI communicator should be split by mpi_init_mdi
-  int use_mpi4py = 0;
-  if ( this_code->language == MDI_LANGUAGE_PYTHON ) {
-    use_mpi4py = 1;
+  // Initialize this code's intra-rank
+  // If using the MPI method, this value may change
+  int mpi_init_flag;
+  if ( MPI_Initialized(&mpi_init_flag) ) {
+    mdi_error("Error in MDI_Init: MPI_Initialized failed");
+    return 1;
+  }
+  if ( mpi_init_flag == 1 && this_code->language != MDI_LANGUAGE_PYTHON ) {
+    MPI_Comm_rank(MPI_COMM_WORLD, &world_rank);
+    MPI_Comm_size(MPI_COMM_WORLD, &world_size);
+    this_code->intra_rank = world_rank;
   }
 
-  if ( strcmp(role, "DRIVER") == 0 ) {
-    // initialize this code as a driver
-
-    if ( strcmp(method, "MPI") == 0 ) {
-      mpi_identify_codes("", use_mpi4py, mpi_communicator);
-      mpi_initialized = 1;
-    }
-    else if ( strcmp(method, "TCP") == 0 ) {
-      if ( has_port == 0 ) {
-	mdi_error("Error in MDI_Init: -port option not provided");
-	return 1;
-      }
-      if ( this_code->intra_rank == 0 ) {
-	tcp_listen(port);
-      }
-      else {
-	// If this isn't rank 0, just set tcp_socket to > 0 so that accept_communicator knows we are using TCP
-	tcp_socket = 1;
-      }
-    }
-    else if ( strcmp(method, "LINK") == 0 ) {
-      //library_initialize();
-    }
-    else if ( strcmp(method, "TEST") == 0 ) {
-      test_initialize();
-    }
-    else {
-      mdi_error("Error in MDI_Init: Method not recognized");
-      return 1;
-    }
-
-  }
-  else if ( strcmp(role,"ENGINE") == 0 ) {
-    // initialize this code as an engine
-
-    if ( strcmp(method, "MPI") == 0 ) {
-      code* this_code = get_code(current_code);
-      mpi_identify_codes(this_code->name, use_mpi4py, mpi_communicator);
-      mpi_initialized = 1;
-    }
-    else if ( strcmp(method, "TCP") == 0 ) {
-      if ( has_hostname == 0 ) {
-	mdi_error("Error in MDI_Init: -hostname option not provided");
-	return 1;
-      }
-      if ( has_port == 0 ) {
-	mdi_error("Error in MDI_Init: -port option not provided");
-	return 1;
-      }
-      if ( this_code->intra_rank == 0 ) {
-	tcp_request_connection(port, hostname);
-      }
-      else {
-	// If this isn't rank 0, just set tcp_socket to > 0 so that accept_communicator knows we are using TCP
-	if ( this_code->intra_rank != 0 ) {
-	  tcp_socket = 1;
-	}
-      }
-    }
-    else if ( strcmp(method, "LINK") == 0 ) {
-      library_initialize();
-    }
-    else if ( strcmp(method, "TEST") == 0 ) {
-      test_initialize();
-    }
-    else {
-      mdi_error("Error in MDI_Init: method not recognized");
-      return 1;
-    }
-
-    
-  }
-  else {
-    mdi_error("Error in MDI_Init: Role not recognized");
+  // Execute the on_selection() function for the user-selected method
+  if ( selected_method->on_selection() ) {
+    mdi_error("MDI method on_selection function failed");
     return 1;
   }
 
@@ -402,33 +314,8 @@ int general_init(const char* options) {
  *
  */
 int general_accept_communicator() {
-  // Give the library method an opportunity to update the current code
-  library_accept_communicator();
-
-  // If MDI hasn't returned some connections, do that now
-  code* this_code = get_code(current_code);
-  if ( this_code->returned_comms < this_code->next_comm - 1 ) {
-    this_code->returned_comms++;
-    return this_code->returned_comms;
-  }
-
-  // Check for any production codes connecting via TCP
-  if ( tcp_socket > 0 ) {
-
-    // Accept a connection via TCP
-    // NOTE: If this is not intra_rank==0, this will always create a dummy communicator
-    tcp_accept_connection();
-
-    // if MDI hasn't returned some connections, do that now
-    if ( this_code->returned_comms < this_code->comms->size ) {
-      this_code->returned_comms++;
-      return (MDI_Comm)this_code->returned_comms;
-    }
-
-  }
-
-  // unable to accept any connections
-  return MDI_COMM_NULL;
+  method* selected_method = get_method(selected_method_id);
+  return selected_method->on_accept_communicator();
 }
 
 
@@ -600,52 +487,34 @@ int general_recv(void* buf, int count, MDI_Datatype datatype, MDI_Comm comm) {
  *                   MDI communicator associated with the intended recipient code.
  */
 int general_send_command(const char* buf, MDI_Comm comm) {
-  // ensure that the driver is the current code
-  library_set_driver_current();
-
-  communicator* this = get_communicator(current_code, comm);
-  int method = this->method;
+  code* this_code = get_code(current_code);
+  method* selected_method = get_method(selected_method_id);
 
   int count = MDI_COMMAND_LENGTH;
-  char* command = malloc( MDI_COMMAND_LENGTH * sizeof(char) );
+  //const char* command = buf;
   int ret = 0;
+  int skip_flag = 0;
 
-  snprintf(command, COMMAND_LENGTH, "%s", buf);
-  if ( method == MDI_LINK ) {
-    // set the command for the engine to execute
-    library_set_command(command, comm);
-
-    if ( command[0] == '<' ) {
-      // execute the command, so that the data from the engine can be received later by the driver
-      ret = library_execute_command(comm);
-      if ( ret != 0 ) {
-	mdi_error("Error in MDI_Send_Command: Unable to execute receive command through library");
-	return ret;
-      }
-    }
-    else if ( command[0] == '>' ) {
-      // flag the command to be executed after the next call to MDI_Send
-      library_data* libd = (library_data*) this->method_data;
-      libd->execute_on_send = 1;
-    }
-    else if ( plugin_mode && ( strcmp( command, "EXIT" ) == 0 || command[0] == '@' ) ) {
-      // this command should be received by MDI_Recv_command, rather than through the execute_command callback
-      ret = general_send( command, count, MDI_CHAR, comm );
-      if ( ret != 0 ) {
-	mdi_error("Error in MDI_Send_Command: Unable to send command");
-	return ret;
-      }
-    }
-    else {
-      // this is a command that neither sends nor receives data, so execute it now
-      ret = library_execute_command(comm);
-      if ( ret != 0 ) {
-	mdi_error("Error in MDI_Send_Command: Unable to execute command through library");
-	return ret;
-      }
+  // Copy the command
+  char* command = malloc( MDI_COMMAND_LENGTH * sizeof(char) );
+  int ichar;
+  for ( ichar=0; ichar < MDI_COMMAND_LENGTH; ichar++) {
+    command[ichar] = '\0';
+  }
+  if ( this_code->intra_rank == 0 ) {
+    for ( ichar=0; ichar < strlen(buf) && ichar < MDI_COMMAND_LENGTH; ichar++ ) {
+      command[ichar] = buf[ichar];
     }
   }
-  else {
+
+  ret = selected_method->on_send_command(command, comm, &skip_flag);
+  if ( ret != 0 ) {
+    mdi_error("MDI Send Command error: method-specific on_send_command() function failed");
+    return ret;
+  }
+
+  // send the command, unless the method's on_send_command function set the skip_flag
+  if ( ! skip_flag ) {
     ret = general_send( command, count, MDI_CHAR, comm );
     if ( ret != 0 ) {
       mdi_error("Error in MDI_Send_Command: Unable to send command");
@@ -653,18 +522,10 @@ int general_send_command(const char* buf, MDI_Comm comm) {
     }
   }
 
-  // if the command was "EXIT", delete this communicator
-  // if running in plugin mode, the plugin system will delete the communicator instead
-  if ( ! plugin_mode && strcmp( command, "EXIT" ) == 0 ) {
-    delete_communicator(current_code, comm);
-
-    // if MDI called MPI_Init, and there are no more communicators, call MPI_Finalize now
-    if ( initialized_mpi == 1 ) {
-      code* this_code = get_code(current_code);
-      if ( this_code->comms->size == 0 ) {
-	MPI_Finalize();
-      }
-    }
+  ret = selected_method->after_send_command(command, comm);
+  if ( ret != 0 ) {
+    mdi_error("MDI Send Command error: method-specific after_send_command() function failed");
+    return ret;
   }
 
   free( command );
@@ -747,31 +608,12 @@ int general_recv_command(char* buf, MDI_Comm comm) {
   int ret;
   code* this_code = get_code(current_code);
   communicator* this = get_communicator(current_code, comm);
+  method* selected_method = get_method(selected_method_id);
 
-  // if this is a linked library, call the driver's node callback
-  if ( this->method == MDI_LINK ) {
-    int iengine = current_code;
-    communicator* engine_comm = get_communicator(current_code, comm);
-
-    // get the driver code to which this communicator connects
-    library_data* libd = (library_data*) engine_comm->method_data;
-    int idriver = libd->connected_code;
-    code* driver_code = get_code(idriver);
-
-    MDI_Comm driver_comm_handle = library_get_matching_handle(comm);
-    communicator* driver_comm = get_communicator(idriver, driver_comm_handle);
-    library_data* driver_lib = (library_data*) driver_comm->method_data;
-
-    // set the current code to the driver
-    current_code = idriver;
-
-    void* class_obj = driver_lib->driver_callback_obj;
-    ret = driver_lib->driver_node_callback(&driver_lib->mpi_comm, driver_comm_handle, class_obj);
-
-    // set the current code to the engine
-    current_code = iengine;
-
-    //return 0;
+  ret = selected_method->on_recv_command(comm);
+  if ( ret != 0 ) {
+    mdi_error("MDI Recv Command error: method-specific on_recv_command() function failed");
+    return ret;
   }
 
   // only receive on rank 0
@@ -812,8 +654,14 @@ int general_recv_command(char* buf, MDI_Comm comm) {
  */
 int register_node(vector* node_vec, const char* node_name)
 {
+  // only register on rank 0
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(node_name) > COMMAND_LENGTH ) {
+  if ( strlen(node_name) >= COMMAND_LENGTH ) {
     mdi_error("Cannot register node name with length greater than MDI_COMMAND_LENGTH");
     return 1;
   }
@@ -832,7 +680,11 @@ int register_node(vector* node_vec, const char* node_name)
   vector_init(callback_vec, sizeof(char[COMMAND_LENGTH]));
   new_node.commands = command_vec;
   new_node.callbacks = callback_vec;
-  snprintf(new_node.name, COMMAND_LENGTH, "%s", node_name);
+  int ichar;
+  for (ichar = 0; ichar < COMMAND_LENGTH; ichar++) {
+    new_node.name[ichar] = '\0';
+  }
+  snprintf(new_node.name, strlen(node_name)+1, "%s", node_name);
   vector_push_back(node_vec, &new_node);
   return 0;
 }
@@ -851,14 +703,20 @@ int register_node(vector* node_vec, const char* node_name)
  */
 int register_command(vector* node_vec, const char* node_name, const char* command_name)
 {
+  // only register on rank 0
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(node_name) > COMMAND_LENGTH ) {
+  if ( strlen(node_name) >= COMMAND_LENGTH ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
     return 1;
   }
 
   // confirm that the command_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(command_name) > COMMAND_LENGTH ) {
+  if ( strlen(command_name) >= COMMAND_LENGTH ) {
     mdi_error("Cannot register command name with length greater than MDI_COMMAND_LENGTH");
     return 1;
   }
@@ -880,7 +738,11 @@ int register_command(vector* node_vec, const char* node_name, const char* comman
 
   // register this command
   char new_command[COMMAND_LENGTH];
-  snprintf(new_command, COMMAND_LENGTH, "%s", command_name);
+  int ichar;
+  for ( ichar = 0; ichar < COMMAND_LENGTH; ichar++) {
+    new_command[ichar] = '\0';
+  }
+  snprintf(new_command, strlen(node_name)+1, "%s", command_name);
   vector_push_back( target_node->commands, &new_command );
 
   return 0;
@@ -900,14 +762,20 @@ int register_command(vector* node_vec, const char* node_name, const char* comman
  */
 int register_callback(vector* node_vec, const char* node_name, const char* callback_name)
 {
+  // only register on rank 0
+  code* this_code = get_code(current_code);
+  if ( this_code->intra_rank != 0 ) {
+    return 0;
+  }
+
   // confirm that the node_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(node_name) > COMMAND_LENGTH ) {
+  if ( strlen(node_name) >= COMMAND_LENGTH ) {
     mdi_error("Node name is greater than MDI_COMMAND_LENGTH");
     return 1;
   }
 
   // confirm that the callback_name size is not greater than MDI_COMMAND_LENGTH
-  if ( strlen(callback_name) > COMMAND_LENGTH ) {
+  if ( strlen(callback_name) >= COMMAND_LENGTH ) {
     mdi_error("Cannot register callback name with length greater than MDI_COMMAND_LENGTH");
     return 1;
   }
@@ -929,7 +797,11 @@ int register_callback(vector* node_vec, const char* node_name, const char* callb
 
   // register this callback
   char new_callback[COMMAND_LENGTH];
-  snprintf(new_callback, COMMAND_LENGTH, "%s", callback_name);
+  int ichar;
+  for ( ichar=0; ichar < COMMAND_LENGTH; ichar++ ) {
+    new_callback[ichar] = '\0';
+  }
+  snprintf(new_callback, strlen(callback_name)+1, "%s", callback_name);
   vector_push_back( target_node->callbacks, &new_callback );
 
   return 0;
@@ -971,16 +843,16 @@ int send_command_list(MDI_Comm comm) {
     // add the name of this node to the list
     node* this_node = vector_get(this_code->nodes, inode);
     int length = (int)strlen(this_node->name);
-    snprintf(&commands[ islot * stride ], COMMAND_LENGTH, "%s", this_node->name);
+    snprintf(&commands[ islot * stride ], strlen(this_node->name)+1, "%s", this_node->name);
     int ichar;
     for (ichar = length; ichar < stride-1; ichar++) {
-      snprintf(&commands[ islot * stride + ichar ], sizeof(char*), "%c", ' ');
+      commands[ islot * stride + ichar ] = ' ';
     }
     if ( this_node->commands->size > 0 ) {
-      snprintf(&commands[ islot * stride + stride - 1 ], sizeof(char*), "%c", ',');
+      commands[ islot * stride + stride - 1 ] = ',';
     }
     else {
-      snprintf(&commands[ islot * stride + stride - 1 ], sizeof(char*), "%c", ';');
+      commands[ islot * stride + stride - 1 ] = ';';
     }
     islot++;
 
@@ -990,13 +862,13 @@ int send_command_list(MDI_Comm comm) {
       length = (int)strlen(command);
       snprintf(&commands[ islot * stride ], COMMAND_LENGTH, "%s", command);
       for (ichar = length; ichar < stride-1; ichar++) {
-	snprintf(&commands[ islot * stride + ichar ], sizeof(char*), "%c", ' ');
+	commands[ islot * stride + ichar ] = ' ';
       }
       if ( icommand == this_node->commands->size - 1 ) {
-	snprintf(&commands[ islot * stride + stride - 1 ], sizeof(char*), "%c", ';');
+	commands[ islot * stride + stride - 1 ] = ';';
       }
       else {
-	snprintf(&commands[ islot * stride + stride - 1 ], sizeof(char*), "%c", ',');
+	commands[ islot * stride + stride - 1 ] = ',';
       }
       islot++;
     }
@@ -1036,6 +908,10 @@ int send_callback_list(MDI_Comm comm) {
   // allocate memory for the callbacks list
   int count = ( ncallbacks + nnodes ) * stride;
   char* callbacks = malloc( count * sizeof(char) );
+  int ichar;
+  for (ichar = 0; ichar < count; ichar++) {
+    callbacks[ichar] = '\0';
+  }
 
   // form the list of callbacks
   int islot = 0;
@@ -1043,16 +919,16 @@ int send_callback_list(MDI_Comm comm) {
     // add the name of this node to the list
     node* this_node = vector_get(this_code->nodes, inode);
     int length = (int)strlen(this_node->name);
-    snprintf(&callbacks[ islot * stride ], COMMAND_LENGTH, "%s", this_node->name);
+    snprintf(&callbacks[ islot * stride ], strlen(this_node->name)+1, "%s", this_node->name);
     int ichar;
     for (ichar = length; ichar < stride-1; ichar++) {
-      snprintf(&callbacks[ islot * stride + ichar ], sizeof(char*), "%c", ' ');
+      callbacks[ islot * stride + ichar ] = ' ';
     }
     if ( this_node->callbacks->size > 0 ) {
-      snprintf(&callbacks[ islot * stride + stride - 1 ], sizeof(char*), "%c", ',');
+      callbacks[ islot * stride + stride - 1 ] = ',';
     }
     else {
-      snprintf(&callbacks[ islot * stride + stride - 1 ], sizeof(char*), "%c", ';');
+      callbacks[ islot * stride + stride - 1 ] = ';';
     }
     islot++;
 
@@ -1062,13 +938,13 @@ int send_callback_list(MDI_Comm comm) {
       length = (int)strlen(callback);
       snprintf(&callbacks[ islot * stride ], COMMAND_LENGTH, "%s", callback);
       for (ichar = length; ichar < stride-1; ichar++) {
-	snprintf(&callbacks[ islot * stride + ichar ], sizeof(char*), "%c", ' ');
+	callbacks[ islot * stride + ichar ] = ' ';
       }
       if ( icallback == this_node->callbacks->size - 1 ) {
-	snprintf(&callbacks[ islot * stride + stride - 1 ], sizeof(char*), "%c", ';');
+	callbacks[ islot * stride + stride - 1 ] = ';';
       }
       else {
-	snprintf(&callbacks[ islot * stride + stride - 1 ], sizeof(char*), "%c", ',');
+	callbacks[ islot * stride + stride - 1 ] = ',';
       }
       islot++;
     }
@@ -1107,12 +983,16 @@ int send_node_list(MDI_Comm comm) {
     // add the name of this node to the list
     node* this_node = vector_get(this_code->nodes, inode);
     int length = (int)strlen(this_node->name);
-    snprintf(&node_list[ inode * stride ], COMMAND_LENGTH, "%s", this_node->name);
+    if ( strlen(this_node->name) >= COMMAND_LENGTH ) {
+      mdi_error("Error in send_node_list: Node name is larger than MDI_COMMAND_LENGTH");
+      return 1;
+    }
+    snprintf(&node_list[ inode * stride ], strlen(this_node->name)+1, "%s", this_node->name);
     int ichar;
     for (ichar = length; ichar < stride-1; ichar++) {
-      snprintf(&node_list[ inode * stride + ichar ], COMMAND_LENGTH, "%c", ' ');
+      node_list[ inode * stride + ichar ] = ' ';
     }
-    snprintf(&node_list[ inode * stride + stride - 1 ], COMMAND_LENGTH, "%c", ',');
+    node_list[ inode * stride + stride - 1 ] = ',';
   }
 
   int ret = general_send( node_list, count, MDI_CHAR, comm );
@@ -1240,14 +1120,16 @@ int get_node_info(MDI_Comm comm) {
     else {
       name_length = (int)(name_end - name_start);
     }
-    if ( name_length > MDI_COMMAND_LENGTH ) {
+    if ( name_length >= MDI_COMMAND_LENGTH ) {
       mdi_error("Error obtaining node information: could not parse node name");
       return 1;
     }
 
     // construct the name of the node
     char* node_name = malloc( MDI_COMMAND_LENGTH * sizeof(char) );
-    snprintf(node_name, COMMAND_LENGTH, "%s", name_start);
+    for (ichar = 0; ichar < name_length; ichar++) {
+      node_name[ichar] = name_start[ichar];
+    }
     for (ichar = name_length; ichar < MDI_COMMAND_LENGTH; ichar++) {
       node_name[ichar] = '\0';
     }
@@ -1283,22 +1165,23 @@ int get_node_info(MDI_Comm comm) {
     else {
       name_length = (int)(name_end - name_start);
     }
-    if ( name_length > MDI_COMMAND_LENGTH ) {
+    if ( name_length >= MDI_COMMAND_LENGTH ) {
       mdi_error("Error obtaining node information: could not parse command name");
       return 1;
     }
 
-    // construct the name
+    // construct the name of the command
     char* command_name = malloc( MDI_COMMAND_LENGTH * sizeof(char) );
-    snprintf(command_name, COMMAND_LENGTH, "%s", name_start);
+    for (ichar = 0; ichar < name_length; ichar++) {
+      command_name[ichar] = name_start[ichar];
+    }
     for (ichar = name_length; ichar < MDI_COMMAND_LENGTH; ichar++) {
       command_name[ichar] = '\0';
     }
-    //printf("DRIVER COMMAND: %d %d %s\n",inode,name_length,command_name);
 
     if ( node_flag == 1 ) { // node
       // store the name of the current node
-      snprintf(current_node, COMMAND_LENGTH, "%s", command_name);
+      snprintf(current_node, strlen(command_name)+1, "%s", command_name);
     }
     else { // command
       // register this command
@@ -1347,14 +1230,16 @@ int get_node_info(MDI_Comm comm) {
     else {
       name_length = (int)(name_end - name_start);
     }
-    if ( name_length > MDI_COMMAND_LENGTH ) {
+    if ( name_length >= MDI_COMMAND_LENGTH ) {
       mdi_error("Error obtaining node information: could not parse callback name");
       return 1;
     }
 
     // construct the name
     char* callback_name = malloc( MDI_COMMAND_LENGTH * sizeof(char) );
-    snprintf(callback_name, COMMAND_LENGTH, "%s", name_start);
+    for (ichar = 0; ichar < name_length; ichar++) {
+      callback_name[ichar] = name_start[ichar];
+    }
     for (ichar = name_length; ichar < MDI_COMMAND_LENGTH; ichar++) {
       callback_name[ichar] = '\0';
     }
@@ -1362,7 +1247,7 @@ int get_node_info(MDI_Comm comm) {
 
     if ( node_flag == 1 ) { // node
       // store the name of the current node
-      snprintf(current_node, COMMAND_LENGTH, "%s", callback_name);
+      snprintf(current_node, strlen(callback_name)+1, "%s", callback_name);
     }
     else { // callback
       // register this callback
