@@ -20,6 +20,12 @@
  * Typically, this will only include a single code, unless the communication method is LIBRARY */
 vector codes;
 
+/*! \brief Vector containing all supported methods */
+vector methods;
+
+/*! \brief ID of the method being used for inter-code communication */
+int selected_method_id = 0;
+
 /*! \brief Index of the active code */
 int current_code = 0;
 
@@ -274,9 +280,18 @@ int new_code() {
   new_code.intra_MPI_comm = MPI_COMM_WORLD;
   new_code.language = MDI_LANGUAGE_C;
 
+  // initialize the name and role strings
+  int ichar;
+  for (ichar=0; ichar < NAME_LENGTH; ichar++) {
+    new_code.name[ichar] = '\0';
+    new_code.role[ichar] = '\0';
+  }
+
   // initialize the character buffer for the plugin path
   new_code.plugin_path = malloc(PLUGIN_PATH_LENGTH * sizeof(char));
-  new_code.plugin_path[0] = '\0';
+  for (ichar=0; ichar < PLUGIN_PATH_LENGTH; ichar++) {
+    new_code.plugin_path[ichar] = '\0';
+  }
 
   // initialize the node vector
   vector* node_vec = malloc(sizeof(vector));
@@ -373,6 +388,70 @@ int delete_code(int code_id) {
 }
 
 
+/*! \brief Create a new method structure and add it to the vector of methods
+ * Returns the handle of the new method
+ */
+int new_method(int method_id) {
+  method new_method;
+  new_method.id = (int)methods.size;
+  new_method.method_id = method_id;
+
+  vector_push_back( &methods, &new_method );
+
+  return new_method.id;
+}
+
+
+/*! \brief Get a method from a method handle
+ * Returns a pointer to the method
+ */
+method* get_method(int method_id) {
+  // Search through all of the codes for the one that matches code_id
+  int imethod;
+  for (imethod = 0; imethod < methods.size; imethod++ ) {
+    method* this_method = vector_get(&methods, imethod);
+    if ( this_method->method_id == method_id ) {
+      return this_method;
+    }
+  }
+  mdi_error("Method not supported for this build of the MDI Library");
+  return NULL;
+}
+
+
+/*! \brief Delete a method
+ * Returns 0 on success
+ */
+int delete_method(int method_id) {
+  method* this_method = get_method(method_id);
+
+  // Search through all of the methods for the one that matches method
+  int imethod;
+  int method_index;
+  int method_found = 0;
+  for (imethod = 0; imethod < methods.size; imethod++ ) {
+    method* method = vector_get(&methods, imethod);
+    if ( method->method_id == method_id ) {
+      method_found = 1;
+      method_index = imethod;
+
+      // stop searching
+      imethod = (int)methods.size;
+    }
+  }
+  if ( method_found != 1 ) {
+    mdi_error("Method not found during delete");
+    return 1;
+  }
+
+  // delete the data for this method from the global vector of methods
+  vector_delete(&methods, method_index);
+
+  return 0;
+}
+
+
+
 /*! \brief Create a new communicator structure and add it to the list of communicators
  * Returns the handle of the new communicator
  */
